<?php

namespace Oro\Bundle\PricingBundle\Tests\Unit\Api\PriceRule\Processor;

use Oro\Bundle\ApiBundle\Tests\Unit\Processor\FormProcessorTestCase;
use Oro\Bundle\PricingBundle\Api\PriceRule\Processor\UpdateLexemesPriceRuleProcessor;
use Oro\Bundle\PricingBundle\Entity\PriceList;
use Oro\Bundle\PricingBundle\Entity\PriceRule;
use Oro\Bundle\PricingBundle\Handler\PriceRuleLexemeHandler;

class UpdateLexemesPriceRuleProcessorTest extends FormProcessorTestCase
{
    /**
     * @var PriceRuleLexemeHandler|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $priceRuleLexemesHandler;

    /**
     * @var UpdateLexemesPriceRuleProcessor
     */
    protected $processor;

    protected function setUp()
    {
        parent::setUp();

        $this->priceRuleLexemesHandler = $this->createMock(PriceRuleLexemeHandler::class);

        $this->processor = new UpdateLexemesPriceRuleProcessor($this->priceRuleLexemesHandler);
    }

    public function testProcess()
    {
        $priceListMock = $this->createPriceListMock();
        $priceRuleMock = $this->createPriceRuleMock($priceListMock);

        $this->priceRuleLexemesHandler
            ->expects(static::once())
            ->method('updateLexemes')
            ->with($priceListMock);

        $this->context->setResult($priceRuleMock);
        $this->processor->process($this->context);
    }

    public function testProcessWithNullResult()
    {
        $this->priceRuleLexemesHandler
            ->expects(static::never())
            ->method('updateLexemes');

        $this->processor->process($this->context);
    }

    public function testProcessWithNullPriceList()
    {
        $priceRuleMock = $this->createPriceRuleMock();

        $this->priceRuleLexemesHandler
            ->expects(static::never())
            ->method('updateLexemes');

        $this->context->setResult($priceRuleMock);
        $this->processor->process($this->context);
    }

    /**
     * @param PriceList|null $priceList
     *
     * @return PriceRule|\PHPUnit_Framework_MockObject_MockObject
     */
    protected function createPriceRuleMock(PriceList $priceList = null)
    {
        $priceRuleMock = $this->createMock(PriceRule::class);

        if (null === $priceList) {
            return $priceRuleMock;
        }

        $priceRuleMock
            ->expects(static::any())
            ->method('getPriceList')
            ->willReturn($priceList);

        return $priceRuleMock;
    }

    /**
     * @return PriceList|\PHPUnit_Framework_MockObject_MockObject
     */
    protected function createPriceListMock()
    {
        return $this->createMock(PriceList::class);
    }
}
